///////////////////////////////////////////////////////////////////////////////
// This file is part of the PngOptimizerCL application
// Copyright (C) 2002/2011 Hadrien Nilsson - psydk.org
//
// PngOptimizerCL is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// 
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

using namespace chustd;

int g_optStatus = 0;

class POApplicationConsole
{
public:
	void OnEngineProgressing(const String& str, enum POEngine::TextColor textColor)
	{
		Console::Write(str);

		if( textColor == POEngine::tcActionFail )
		{
			g_optStatus = 1;
		}
	}
};

int main()
{
	String strCmdLine = Process::GetCommandLine();

	ArgvParser ap(strCmdLine);
	if( !ap.GetFlag(L"file") )
	{
		// Display usage
		String version = "PngOptimizerCL 2.0";
#if defined(_M_X64)
		version = version + String(" (x64)");
#elif defined(_M_IX86)  
		version = version + String(" (x86)");
#endif

		Console::WriteLine(version + " (C) 2002/2011 by Hadrien Nilsson - psydk.org");
		Console::WriteLine("Converts GIF, BMP and TGA files to optimized PNG files.");
		Console::WriteLine("Optimizes and cleans PNG files.");
		Console::WriteLine("");
		Console::WriteLine("Usage:  PngOptimizerCL -file:\"yourfile.png\" ");
		POEngineSettings::WriteArgvUsage("  ");
		Console::WriteLine("");
		Console::WriteLine("Values enclosed with [] are optional.");
		Console::WriteLine("Chunk option meaning: R=Remove, K=Keep, F=Force. 0|1|2 can be used too.");
		Console::WriteLine("");
		Console::WriteLine("You can use wildcards to convert/optimize several files at once.");
		Console::WriteLine("Only files which extension is 'png', 'gif', 'bmp' or 'tga' will be handled.");
		Console::WriteLine("This example handles all PNG, GIF, BMP and TGA files of the current directory:");
		Console::WriteLine("");
		Console::WriteLine("  PngOptimizerCL -file:\"*.*\"");
		Console::WriteLine("");
	
		Console::Write(L"Press any key to continue");
		Console::WaitForKey();

		return 0;
	}

	POApplicationConsole app;

	// PngOptimizer.exe -file:"myfile.png"
	// Console mode

	POEngine engine;
	engine.m_eventProgressing.Handle(&app, &POApplicationConsole::OnEngineProgressing);

	engine.m_settings.LoadFromArgv(ap);

	//////////////////////////////////////////////////////////////////
	String filePath = ap.GetFlagString(L"file");
	String dir, fileName;
	FilePath::Split(filePath, dir, fileName);

	StringArray fileNames = Directory::GetFileNames(dir, fileName);
	if( fileNames.IsEmpty() )
	{
		Console::WriteLine("File not found: " + filePath);
		return 1;
	}
	engine.OptimizeFiles(dir, fileNames);
	
	return g_optStatus;
}
