///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2011 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Console.h"
#include "String.h"

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////

Console::Console()
{

}

Console::~Console()
{

}

void Console::WriteLine(const chustd::String& str)
{
	Write(str + L"\r\n");
}

void Console::Write(const chustd::String& str)
{
#ifdef WIN32
	// Windows functions
	HANDLE hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

	DWORD charWrittenCount = 0;
	DWORD length = str.GetLength();
	BOOL bOk = WriteConsoleW(hStdOut, str.GetBuffer(), length, &charWrittenCount, NULL);
	bOk;
#elif CHUOS

#endif
}



bool Console::WaitForInput(chustd::String& str)
{
	static wchar szPrompt[] = L">";
	return WaitForInput(szPrompt, str);
}

// Wait for a string input by the user
// strPrompt : symbol to print before waiting for the string
// strInput : result
bool Console::WaitForInput(const chustd::String& strPrompt, chustd::String& strInput)
{
	Write(strPrompt);

	wchar szBuffer[128];

#ifdef WIN32
	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);

	DWORD numberOfCharsToRead = (sizeof(szBuffer) / 2) - 1;
	DWORD numberOfCharsRead = 0;

	BOOL bWinOk = ReadConsoleW(hStdIn, szBuffer, numberOfCharsToRead, &numberOfCharsRead, NULL);

	if( !bWinOk )
		return false;
#else

	int32 numberOfCharsRead = 0;
#endif

	// Removes any carriage return or line feed
	int32 i = numberOfCharsRead - 1;
	for(; i >= 0; --i)
	{
		wchar c = szBuffer[i];
		if( c != 13 && c != 10 )
			break;
	}

	int32 realCharCount = i + 1;
	strInput.SetBuffer(szBuffer, realCharCount);
	return true;
}

uint32 Console::WaitForKey()
{
	uint32 char32 = 0;

#ifdef WIN32
	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);

	// Switch to raw mode (no line input, no echo input)
	DWORD oldMode = 0;
	GetConsoleMode(hStdIn, &oldMode);
	SetConsoleMode(hStdIn, 0);
	
	for(;;)
	{
		DWORD read = 0;
		INPUT_RECORD inputrec;
		if( !ReadConsoleInputW(hStdIn, &inputrec, 1, &read) || (read == 0) )
		{
			char32 = 0;
			break;
		}
		
		if( (inputrec.EventType == KEY_EVENT) && inputrec.Event.KeyEvent.bKeyDown )
		{
			char32 = inputrec.Event.KeyEvent.uChar.UnicodeChar;
			if( char32 != 0 )
			{
				break;
			}
		}
	}
	
	SetConsoleMode(hStdIn, oldMode);
#endif
	return char32;
}
