///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2011 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_PTRARRAY_H
#define CHUSTD_PTRARRAY_H

#include "Array.h"

namespace chustd {

// Simple array of pointers
template <class T> class PtrArrayNoDelete : public Array<void*>
{
public:
	T*& operator [] (int32 index)
	{
		return (T*&) (m_paTs[index]);
	}

	T* operator [] (int32 index) const
	{
		return (T*) (m_paTs[index]);
	}
};


// Manages a pointers array
// The array owns the data, so it deletes the pointed data
template <class T> class PtrArray : public Array<T*>
{
public:
	int32 Add(T* pT)
	{
		// Every address must be unique
		if(pT)
		{
			ASSERT( Find(pT) < 0);
		}

		return Array<T*>::Add(pT);
	}

	int32 Add(T* const * papTs, int32 count) {	return Array<T*>::Add(papTs, count); }
	int32 Add(const PtrArray<T>& arr) { return Array<T*>::Add(arr); }

	PtrArray<T>& operator = (const PtrArray<T>& arr);

	PtrArray();
	PtrArray(const PtrArray<T>& arr);
	virtual ~PtrArray();

protected:
	virtual void DeletePointedData(int32 startIndex, int32 count);
};

template <class T>
PtrArray<T>::PtrArray()
{
}

// Copy constructor
template <class T>
PtrArray<T>::PtrArray(const PtrArray<T>& arr)
{
	if( arr.IsReturnedByValue() )
	{
		this->m_paTs = arr.m_paTs;
		this->m_count = arr.m_count;
		this->m_nRbvAndAllocated = arr.GetCapacity(); // Set the rbv bit to 0

		// Const cast, bad, but well, advandced C++
		PtrArray<T>& arr2 = (PtrArray<T>&) arr;

		arr2.m_paTs = null;
		arr2.m_count = 0;
		arr2.m_nRbvAndAllocated = 0;
	}
	else
	{
		this->m_paTs = null;
		this->m_count = 0;
		this->m_nRbvAndAllocated = 0;

		Add(arr.m_paTs, arr.m_count);

		// Create copies of pointed data
		for(int i = 0; i < this->m_count; i++)
		{
			this->m_paTs[i] = new T( *this->m_paTs[i]);
		}
	}
}


// Assignment operator
template <class T>
PtrArray<T>& PtrArray<T>::operator = (const PtrArray<T>& arr)
{
	Array<T*>::operator = (arr);
	
	// Create copies of pointed data
	for(int i = 0; i < this->m_count; i++)
	{
		this->m_paTs[i] = new T( *this->m_paTs[i]);
	}

	return *this;
}

template <class T>
PtrArray<T>::~PtrArray()
{
	for(int i = 0; i < this->m_count; ++i)
	{
		delete this->m_paTs[i];
	}
}

template <class T>
void PtrArray<T>::DeletePointedData(int32 startIndex, int32 count)
{
	for(int i = 0; i < count; i++)
	{
		T* pElement = this->m_paTs[startIndex + i];
		delete pElement;
	}
}

} // namespace chustd

#endif // ndef CHUSTD_PTRARRAY_H
