///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2011 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TcpClientSocket.h"
#include "Array.h"
#include "TextEncoding.h"
#include "Dns.h"

using namespace chustd;

//////////////////////////////////////////////////////////////////////

bool TcpClientSocket::AsyncConnect(const String& strServerName, uint16 port, PFN_ASYNCHANDLER /*pfnAsyncHandler*/)
{
	if( !IsOpen() )
	{
		// Socket not created
	
		// Auto creation
		if( !super::Open(Socket::typeTcp) )
			return false;
	}

#ifdef WIN32
	// Questions pendantes :
	// 1. O est-ce qu'on prend le hWnd ?
	//   - on cre une fentre cache pour les sockets (mthodes MFC) ;
	//   - on utilise la fentre de plus haut niveau (plus sympa mais difficulut pour programme Console).
	// 2. Comment on gre le WM_ envoy lors d'un vnement rseau ?
	//   - un seul vnement pour toutes les sockets ;
	//   - un vnement par socket (limite du nombre de socket ?) ;
	//   - un vnement WM_ par type d'vnement FD_<-- Ne fonctionne pas (cf Doc MSDN)

	HWND hWnd = 0;
	const int WM_APP_ONNETWORKEVENT = WM_APP + 4;

	int64 socketId = GetSocketID();
	WSAAsyncSelect( SOCKET_ID(socketId), hWnd, WM_APP_ONNETWORKEVENT, FD_CONNECT);
#endif

	return Connect2(strServerName, port, true);
}

bool TcpClientSocket::Connect(const String& strServerName, uint16 port)
{
	if( !IsOpen() )
	{
		// Socket not created
	
		// Auto creation
		if( !super::Open(Socket::typeTcp) )
			return false;
	}
	
	return Connect2(strServerName, port, false);
}

// Returns true upon success
bool TcpClientSocket::Connect2(const String& strServerName, uint16 port, bool bAsync)
{
	// gethostbyname is sloooow even with IP addresses, so first let's use our own fast routine
	// to check if the address is an IP address
	uint32 nIpAddress;
	if( !StringToIpAddress(strServerName, nIpAddress) )
	{
		Dns::HostEntry he;
		if( !Dns::GetHostByName(strServerName, he) )
			return false;
		
		nIpAddress = he.m_aIpAddresses[0];
	}

#ifdef WIN32
	// Setup server features we want to connect to
	struct sockaddr_in sai_server;

	// nIpAddress byte order is plateform dependant, convert it to network byte order (big endian)
	sai_server.sin_addr.s_addr = htonl(nIpAddress);
	sai_server.sin_family = AF_INET; // Network family
	sai_server.sin_port = htons(port);
	uint32* paZero = (uint32*) sai_server.sin_zero;
	paZero[0] = paZero[1] = 0;

	const int ret = connect( SOCKET_ID(m_nIDSocket), (struct sockaddr*)&sai_server, sizeof(sai_server));
	if( bAsync )
	{
		if( ret != 0 && ::GetLastError() != WSAEWOULDBLOCK )
		{
			// Real error
			return false;
		}
		return true;
	}
	
	bool bOk = ret == 0;
	return bOk;

#elif CHUOS
	return false;
#endif
}
