///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2011 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ZStream.h"
#include "Memory.h"

#include "File.h"
#include "StringBuilder.h"
#include "Console.h"

// inflate / deflate (PNG)
#include "zlib/zlib.h"

//////////////////////////////////////////////////////////////////////
namespace chustd {
class ZStreamImpl
{
public:
	z_stream m_ZLibStream;
};
}

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////

ZStream::ZStream()
{
	m_pImpl = new ZStreamImpl;
	
	Memory::Zero(&m_pImpl->m_ZLibStream, sizeof(m_pImpl->m_ZLibStream));
}

ZStream::~ZStream()
{
	delete m_pImpl;
}

void ZStream::SetBuffers(const uint8* pInNext, uint32 nInAvailable, uint8* pOutNext, uint32 outAvailable)
{
	m_pImpl->m_ZLibStream.next_in = (unsigned char*) pInNext;
	m_pImpl->m_ZLibStream.avail_in = nInAvailable;
	
	m_pImpl->m_ZLibStream.next_out = (unsigned char*) pOutNext;
	m_pImpl->m_ZLibStream.avail_out = outAvailable;
}

uint32 ZStream::GetOutAvailable() const
{
	return m_pImpl->m_ZLibStream.avail_out;
}

uint32 ZStream::GetOutTotalRead() const
{
	return m_pImpl->m_ZLibStream.total_out;
}

///////////////////////
// Compression

int ZStream::DeflateInit(int level)
{
	return deflateInit(&m_pImpl->m_ZLibStream, level);
}
int ZStream::DeflateInit2(int level, uint8 method, int windowBits, int memLevel, int strategy)
{
	return deflateInit2(&m_pImpl->m_ZLibStream, level, method, windowBits, memLevel, strategy);
}
int ZStream::Deflate(int flush)
{
	return deflate(&m_pImpl->m_ZLibStream, flush);
}
int ZStream::DeflateParams(int newLevel, int strategy)
{
	return deflateParams(&m_pImpl->m_ZLibStream, newLevel, strategy);
}
int ZStream::DeflateEnd()
{
	return deflateEnd(&m_pImpl->m_ZLibStream);
}

int ZStream::DeflateSetDictionary(const uint8* pDictionary, uint32 dictLength)
{
	return deflateSetDictionary(&m_pImpl->m_ZLibStream, (const unsigned char*) pDictionary, dictLength);
}
int ZStream::DeflateReset()
{
	return deflateReset(&m_pImpl->m_ZLibStream);
}

uint32 ZStream::DeflateBound(uint32 sourceLength)
{
	return deflateBound(0, sourceLength);
}
	
///////////////////////
// Uncompression
int ZStream::InflateInit()
{
	return inflateInit(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateInit2(int windowBits)
{
	return inflateInit2(&m_pImpl->m_ZLibStream, windowBits);
}

int ZStream::Inflate(int flushType)
{
	return inflate(&m_pImpl->m_ZLibStream, flushType);
}
int ZStream::InflateEnd()
{
	return inflateEnd(&m_pImpl->m_ZLibStream);
}

int ZStream::InflateSetDictionary(const uint8* pDictionary, uint32 dictLength)
{
	return inflateSetDictionary(&m_pImpl->m_ZLibStream, (const unsigned char*) pDictionary, dictLength);
}
int ZStream::InflateSync()
{
	return inflateSync(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateReset()
{
	return inflateReset(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateSyncPoint()
{
	return inflateSyncPoint(&m_pImpl->m_ZLibStream);
}

// Compresses the source buffer into the destination buffer.  sourceLen is
// the byte length of the source buffer. Upon entry, destLen is the total
// size of the destination buffer, which must be at least the value returned
// by compressBound(sourceLen). Upon exit, destLen is the actual size of the
// compressed buffer.
//
// This function can be used to compress a whole file at once if the
// input file is mmap'ed.
//
// compress returns Z_OK if success, Z_MEM_ERROR if there was not
// enough memory, Z_BUF_ERROR if there was not enough room in the output buffer.

int ZStream::Compress(uint8* pDest, uint32* pDestLen, const uint8* pSource, uint32 sourceLen, int level)
{
	unsigned long destLen = *pDestLen;
	int ret = compress2( (unsigned char*) pDest, &destLen, (const unsigned char*) pSource, sourceLen, level);
	
	*pDestLen = destLen;

	return ret;
}

const char* ZStream::GetLastError() const
{
	if( m_pImpl == null )
	{
		return "";
	}
	
	const char* psz = m_pImpl->m_ZLibStream.msg;
	if( psz == null )
	{
		return "";
	}
	return psz;
}

