#include "stdafx.h"

bool operator==(const POEngineSettings& s1, const POEngineSettings& s2)
{
	bool ret = true;
	ret &= s1.backupOldPngFiles == s2.backupOldPngFiles;
	ret &= s1.keepInterlacing == s2.keepInterlacing;
	ret &= s1.avoidGreyWithSimpleTransparency == s2.avoidGreyWithSimpleTransparency;
	ret &= s1.ignoreAnimatedGifs == s2.ignoreAnimatedGifs;
	ret &= s1.keepFileDate == s2.keepFileDate;

	ret &= s1.bkgdOption == s2.bkgdOption;
	ret &= s1.bkgdColor == s2.bkgdColor;

	ret &= s1.textOption == s2.textOption;
	ret &= s1.textKeyword == s2.textKeyword;
	ret &= s1.textData == s2.textData;

	return ret;
}

TEST(POEngineSettings, EmptyArgv)
{
	ArgvParser ap("");
	POEngineSettings settings;
	settings.LoadFromArgv(ap);
	
	POEngineSettings exp;
	exp.backupOldPngFiles = false; // The only difference with INI loading
	ASSERT_TRUE(settings == exp);
}

TEST(POEngineSettings, AllArgv)
{
	ArgvParser ap("-BackupOldPngFiles -KeepInterlacing -AvoidGreyWithSimpleTransparency -IgnoreAnimatedGifs -KeepFileDate "
		"-KeepBackgroundColor -KeepTextualData");
	POEngineSettings settings;
	settings.LoadFromArgv(ap);
	
	POEngineSettings exp;
	exp.backupOldPngFiles = true;
	exp.keepInterlacing = true;
	exp.avoidGreyWithSimpleTransparency = true;
	exp.ignoreAnimatedGifs = true;
	exp.keepFileDate = true;
	exp.bkgdOption = POChunk_Keep;
	exp.textOption = POChunk_Keep;
	ASSERT_TRUE(settings == exp);
}

TEST(POEngineSettings, RemoveChunkArgv)
{
	ArgvParser ap("-KeepBackgroundColor:0 -KeepTextualData:0");
	POEngineSettings settings;
	settings.LoadFromArgv(ap);
	
	POEngineSettings exp;
	exp.backupOldPngFiles = false;
	exp.bkgdOption = POChunk_Remove;
	exp.textOption = POChunk_Remove;
	ASSERT_TRUE(settings == exp);

	ArgvParser apAlt("-KeepBackgroundColor:R -KeepTextualData:R");
	settings.LoadFromArgv(apAlt);
	ASSERT_TRUE(settings == exp);
}

TEST(POEngineSettings, KeepChunkArgv)
{
	ArgvParser ap("-KeepBackgroundColor:1 -KeepTextualData:1");
	POEngineSettings settings;
	settings.LoadFromArgv(ap);
	
	POEngineSettings exp;
	exp.backupOldPngFiles = false;
	exp.bkgdOption = POChunk_Keep;
	exp.textOption = POChunk_Keep;
	ASSERT_TRUE(settings == exp);

	ArgvParser apAlt("-KeepBackgroundColor:K -KeepTextualData:K");
	settings.LoadFromArgv(apAlt);
	ASSERT_TRUE(settings == exp);
}

TEST(POEngineSettings, ForceChunkArgv)
{
	ArgvParser ap(L"-KeepBackgroundColor:2 -ForcedBackgroundColor:a1b2c3 -KeepTextualData:2 -ForcedTextKeyword:toffe -ForcedTextData:\"Soie brode\"");
	POEngineSettings settings;
	settings.LoadFromArgv(ap);
	
	POEngineSettings exp;
	exp.backupOldPngFiles = false;
	exp.bkgdOption = POChunk_Force;
	exp.bkgdColor.SetRgb(0xa1u, 0xb2u, 0xc3u);
	exp.textOption = POChunk_Force;
	exp.textKeyword = L"toffe";
	exp.textData = L"Soie brode";
	ASSERT_TRUE(settings == exp);

	ArgvParser apAlt(L"-KeepBackgroundColor:F -ForcedBackgroundColor:A1B2C3 -KeepTextualData:F -ForcedTextKeyword:toffe -ForcedTextData:\"Soie brode\"");
	settings.LoadFromArgv(apAlt);
	ASSERT_TRUE(settings == exp);
}






