// UT_ASSERTChustd.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

TEST(ScanType, Int32)
{
	// Cas gnral
	int32 nAdvance = 0;
	int32 nResult = 0;
	bool bRet = ScanType(L"123", nAdvance, 'd', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 3);
	ASSERT_TRUE(nResult == 123);

	// Avec espaces
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L" \t\n\r 123", nAdvance, 'd', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 8);
	ASSERT_TRUE(nResult == 123);

	// Avec signe +
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L"+123", nAdvance, 'd', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 4);
	ASSERT_TRUE(nResult == 123);

	// Avec signe -
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L"-123", nAdvance, 'd', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 4);
	ASSERT_TRUE(nResult == -123);

	// Avec signe + et une espace
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L"+ 123", nAdvance, 'd', &nResult);
	ASSERT_TRUE(bRet == false);
	ASSERT_TRUE(nAdvance == 0); // Doit rester inchang
	ASSERT_TRUE(nResult == 0); // Doit rester inchang

	// Hexadcimal
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L"aA12cF", nAdvance, 'x', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 6);
	ASSERT_TRUE(nResult == 0x0aA12cF);

	// Binaire
	nAdvance = 0;
	nResult = 0;
	bRet = ScanType(L"10110001", nAdvance, 'b', &nResult);
	ASSERT_TRUE(bRet == true);
	ASSERT_TRUE(nAdvance == 8);
	ASSERT_TRUE(nResult == 0x00B1);
}

TEST(ByteArrayTest, Find)
{
	ByteArray aData;
	aData.Add('x');
	aData.Add('a');
	ASSERT_TRUE(aData.Find((uint8*)"z", 2) == -1);
	ASSERT_TRUE(aData.Find((uint8*)"x", 0) == 0);
	ASSERT_TRUE(aData.Find((uint8*)"a", 1) == 1);
	ASSERT_TRUE(aData.Find((uint8*)"xa", 2) == 0);
	ASSERT_TRUE(aData.Find((uint8*)"uvw", 3) == -1);

	aData.Add('b');
	aData.Add('c');
	ASSERT_TRUE(aData.Find((uint8*)"abc", 3) == 1);
}

static int   s_zero      = 0;


int64 Win32_DateFieldsToChuStamp(int32 nYear, int32 nMonth, int32 nDay)
{
	int64 nStamp = 0;

#ifdef WIN32
	SYSTEMTIME st;
	st.wYear = (uint16) nYear;
	st.wMonth = (uint16) nMonth;
	st.wDayOfWeek = 0; // Ignored by SystemTimeToFileTime
	st.wDay = (uint16) nDay;
	st.wHour = 0;
	st.wMinute = 0;
	st.wSecond = 0;
	st.wMilliseconds = 0; // TODO: could be cool to have this

	FILETIME ft;
	BOOL bOk = SystemTimeToFileTime(&st, &ft);
	
	nStamp = ft.dwHighDateTime;
	nStamp <<= 32;
	nStamp |= ft.dwLowDateTime;

	nStamp /= 10000;

#endif

	return nStamp;
}

// Check conversion from year-month-day to timestamp
// Chu timestamps are int64 numbers representing the number of milliseconds since 1601-01-01 00:00:00.000
// We can ASSERT_TRUE our algorithm by using Windows one which uses similar timestamps
TEST(DateTimeTest, FieldsToChuStamp)
{
	DateTime date;
	for(int16 iYear = 1601; iYear <= 2100; ++iYear)
	{
		for(uint8 iMonth = 1; iMonth <= 12; ++iMonth)
		{
			for(uint8 iDay = 1; iDay < 5; ++iDay)
			{
				date.Set(iYear, iMonth, iDay);
				int64 nChuTimeStamp = date.GetChuTimeStamp();
				int64 nWin32ChuTimeStamp = Win32_DateFieldsToChuStamp(iYear, iMonth, iDay);

				ASSERT_TRUE(nChuTimeStamp == nWin32ChuTimeStamp);
			}
		}
	}
}


void Win32_Date_Split(int64 nStamp, int32& nYear, int32& nMonth, int32& nDay)
{
	nStamp *= 10000;

	int32 nHour, nMinute, nSecond;

#ifdef WIN32
	// TODO : recode Win32.FileTimeToSystemTime
	FILETIME ft;
	ft.dwHighDateTime = DWORD(nStamp >> 32);
	ft.dwLowDateTime = DWORD(nStamp & 0x00000000ffffffff);

	SYSTEMTIME st;
	if( !FileTimeToSystemTime(&ft, &st))
	{
		nYear = 0;
		nMonth = 0;
		nDay = 0;
		nHour = 0;
		nMinute = 0;
		nSecond = 0;
	}
	else
	{
		nYear = int16( st.wYear);
		nMonth = uint8( st.wMonth); // January = 1
		nDay = uint8( st.wDay);
		nHour = uint8( st.wHour);
		nMinute = uint8( st.wMinute);
		nSecond = uint8( st.wSecond);
	}
#endif
}

TEST(DateTimeTest, ChuStampToFields)
{
	// 146097 = DaysPer400Years
	int64 nOneDay = 24 * 60 * 60 * 1000;
	int64 nStampMax = 2 * 146097;
	nStampMax *= nOneDay;

	int64 nStampMin = 146097;
	nStampMin *= nOneDay;
	nStampMin -= nOneDay;

	nStampMin = nOneDay * 365;

	for(int64 iStamp = 0; iStamp <= nStampMax; iStamp += nOneDay)
	{
		DateTime date = DateTime::FromChuTimeStamp(iStamp);

		int32 nYear = date.GetYear();
		int32 nMonth = date.GetMonth();
		int32 nDay = date.GetDay();

		int32 nYearWin32, nMonthWin32, nDayWin32;
		Win32_Date_Split(iStamp, nYearWin32, nMonthWin32, nDayWin32);

		//Console::WriteLine( String::FromInt(nYear) + L"-" + String::FromInt(nMonth) + L"-" + String::FromInt(nDay));

		ASSERT_TRUE(nYear == nYearWin32);
		ASSERT_TRUE(nMonth == nMonthWin32);
		ASSERT_TRUE(nDay == nDayWin32);
	}
}

TEST(MathTest, Log2)
{
	int32 n0, n1, n2, n3;

	n0 = Math::Log2(0);
	n1 = Math::Log2(1);
	n2 = Math::Log2(2);
	n3 = Math::Log2(3);
	
	ASSERT_TRUE( n0 == 0);
	ASSERT_TRUE( n1 == 0);
	ASSERT_TRUE( n2 == 1);
	ASSERT_TRUE( n3 == 1);
	
	for(int32 i = 4; i <= 7; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 2);
	}

	for(int32 i = 8; i <= 15; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 3);
	}

	for(int32 i = 16; i <= 31; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 4);
	}

	for(int32 i = 32; i <= 63; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 5);
	}

	for(int32 i = 64; i <= 127; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 6);
	}

	for(int32 i = 128; i <= 255; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 7);
	}

	for(int32 i = 256; i <= 511; ++i)
	{
		ASSERT_TRUE( Math::Log2(i) == 8);
	}
}

class ArrElem
{
public:
	ArrElem()  { m_nValue = 0x11223344; }
	~ArrElem() { m_nValue = 0xaabbccdd; }
public:
	uint32 m_nValue;
};

TEST(ArrayTest, Misc)
{
	////////////////////////////////////////////////////////////////////////////////////////
	// Test array copy big from small, with constructors and destructors correctly called
	Array<ArrElem> a1;
	a1.EnsureCapacity(10);
	
	// We do not use the [] operator so we can read/write after the array size
	a1.GetBuffer()[0].m_nValue = 0xfdfdfdfd;
	a1.GetBuffer()[1].m_nValue = 0xfdfdfdfd;
	a1.GetBuffer()[2].m_nValue = 0xfdfdfdfd;
	a1.GetBuffer()[3].m_nValue = 0xfdfdfdfd;

	a1.SetSize(2);

	ASSERT_TRUE( a1.GetBuffer()[0].m_nValue == 0x11223344 );
	ASSERT_TRUE( a1.GetBuffer()[1].m_nValue == 0x11223344 );
	ASSERT_TRUE( a1.GetBuffer()[2].m_nValue == 0xfdfdfdfd );
	ASSERT_TRUE( a1.GetBuffer()[3].m_nValue == 0xfdfdfdfd );

	Array<ArrElem> a2;
	a2.SetSize(1);
	a2[0].m_nValue = 0x55667788;

	a1 = a2; // Assignment

	ASSERT_TRUE( a1.GetBuffer()[0].m_nValue == 0x55667788 );
	ASSERT_TRUE( a1.GetBuffer()[1].m_nValue == 0xaabbccdd );
	ASSERT_TRUE( a1.GetBuffer()[2].m_nValue == 0xfdfdfdfd );
	ASSERT_TRUE( a1.GetBuffer()[3].m_nValue == 0xfdfdfdfd );
	////////////////////////////////////////////////////////////////////////////////////////
}

TEST(DirectoryTest, Misc)
{
	// We start in a known state
	Directory::Delete(L"C:\\testdir");

	ASSERT_TRUE( !Directory::Exists(L"C:/testdir/") );

	ASSERT_TRUE( Directory::Create(L"C:\\testdir") );
	ASSERT_TRUE( Directory::Exists(L"C:\\testdir") );
	ASSERT_TRUE( Directory::Exists(L"C:\\testdir\\") );
	ASSERT_TRUE( Directory::Exists(L"C:\\testdir/") );
	ASSERT_TRUE( Directory::Exists(L"C:/testdir/") );
	ASSERT_TRUE( Directory::Delete(L"C:\\testdir") );
	ASSERT_TRUE( !Directory::Exists(L"C:/testdir/") );
}

TEST(Color32Test, Misc)
{
	// 0 11111 11110 11100 => 0111_1111 1101_1100 => 7FDC
	uint16 nValue = 0x7FDC;
	Color32 col = Color32::From16bppRgb555(nValue);

	uint8 r, g, b, a;
	col.ToRgba(r, g, b, a);

	// 11111 => 11111111
	ASSERT_TRUE( r == 0xFF );
	
	// 11110 => 11110111
	ASSERT_TRUE( g == 0xF7 );

	// 11100 => 11100111
	ASSERT_TRUE( b == 0xE7 );

	ASSERT_TRUE( a == 0xFF );

	// 0 10011 10111 01011 => 0100 1110 1110 1011 => 0x4EEB
	nValue = 0x4EEB;
	col = Color32::From16bppRgb555(nValue);

	col.ToRgba(r, g, b, a);

	// 10011 => 10011 100
	ASSERT_TRUE( r == 0x9C );
	
	// 10111 => 10111 101
	ASSERT_TRUE( g == 0xBD );

	// 01011 => 01011 010
	ASSERT_TRUE( b == 0x5A );

	ASSERT_TRUE( a == 0xFF );
}

TEST(MemoryTest, Misc)
{
	{
		uint8 a1[5] = { 1, 2, 3, 0x33, 0x44 };
		uint8 a2[5] = {0xdd, 0xee, 0xff, 0xaa, 0xbb };
		
		Memory::Set(a2, 0x11, 3);
		ASSERT_TRUE(a2[0] == 0x11 && a2[1] == 0x11 && a2[2] == 0x11 && a2[3] == 0xaa && a2[4] == 0xbb);
		
		Memory::Copy(a2, a1, 3);
		ASSERT_TRUE(a2[0] == 0x1 && a2[1] == 0x2 && a2[2] == 0x3 && a2[3] == 0xaa && a2[4] == 0xbb);

		Memory::Zero(a2, 3);
		ASSERT_TRUE(a2[0] == 0x0 && a2[1] == 0x0 && a2[2] == 0x0 && a2[3] == 0xaa && a2[4] == 0xbb);
	}

	uint8 a3[6] = {   1,    2,    3,    4, 0x33, 0x44 };
	uint8 a4[6] = {0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee };

	Memory::Set(a4, 0x11, 4);
	ASSERT_TRUE(a4[0] == 0x11 && a4[1] == 0x11 && a4[2] == 0x11 && a4[3] == 0x11 && a4[4] == 0xdd && a4[5] == 0xee);
	
	Memory::Copy(a4, a3, 4);
	ASSERT_TRUE(a4[0] == 0x1 && a4[1] == 0x2 && a4[2] == 0x3 && a4[3] == 0x04 && a4[4] == 0xdd && a4[5] == 0xee);

	Memory::Zero(a4, 4);
	ASSERT_TRUE(a4[0] == 0x0 && a4[1] == 0x0 && a4[2] == 0x0 && a4[3] == 0x0 && a4[4] == 0xdd && a4[5] == 0xee);
}

TEST(RandomTest, Misc)
{
	Random rnd;

	FixArray<int32, 21> aCounters = {0};

	for(int i = 0; i < 200; ++i)
	{
		int32 nVal = rnd.GetNext(0, 20);
		aCounters[nVal]++;
	}
	/*
	foreach(aCounters, iCounter)
	{
		const int32 nPlop = aCounters[iCounter];
		for(int i = 0; i < nPlop; ++i)
		{
			Console::Write(L"#");
		}
		Console::WriteLine(L"");
	}*/
}

TEST(MemIniFileTest, Dump)
{
	MemIniFile ini;

	ini.SetSection(L"Engine");
	ini.SetBool(L"AvoidGreyWithSimpleTransparency", true);
	ini.SetBool(L"BackupOldPngFiles", false);
	ini.SetBool(L"KeepBackgroundColor", false);
	ini.SetBool(L"KeepInterlacing", false);

	ini.SetSection(L"Screenshots");
	ini.SetBool(L"UseDefaultDir", true);
	ini.SetString(L"CustomDir", L"D:\\toile des neiges");
	ini.SetBool(L"AskForFileName", false);
	ini.SetBool(L"MaximizeCompression", false);

	ini.SetSection(L"Window");
	ini.SetInt(L"X", 15);
	ini.SetInt(L"Y", 20);
	ini.SetInt(L"Width", 400);
	ini.SetInt(L"Height", 300);

	ini.SetCommentLine1(L"PngOptimizer configuration file");
	ini.SetCommentLine2(L"This file is encoded in UTF-8");

	ini.Dump(L"test.ini", false);
	ini.Dump(L"test-crlf.ini", true);
}

TEST(MemIniFileTest, Load)
{
	MemIniFile ini;
	ASSERT_TRUE( ini.Load(L"test.ini"));

	ASSERT_TRUE( ini.SetSection(L"Engine"));
	
	bool bAvoidGreyWithSimpleTransparency;
	ASSERT_TRUE( ini.GetBool(L"AvoidGreyWithSimpleTransparency", bAvoidGreyWithSimpleTransparency));
	ASSERT_TRUE( bAvoidGreyWithSimpleTransparency == true);

	bool bBackupOldPngFiles;
	ASSERT_TRUE( ini.GetBool(L"BackupOldPngFiles", bBackupOldPngFiles));
	ASSERT_TRUE( bBackupOldPngFiles == false);

	ASSERT_TRUE( ini.SetSection(L"Screenshots"));
	
	String strCustomDir;
	ASSERT_TRUE( ini.GetString(L"CustomDir", strCustomDir));
	ASSERT_TRUE( strCustomDir == L"D:\\toile des neiges");

	ASSERT_TRUE( ini.SetSection(L"Window"));
	
	int32 nX;
	ASSERT_TRUE( ini.GetInt(L"X", nX));
	ASSERT_TRUE( nX == 15);
}

TEST(TextEncodingTest, Misc)
{
	ByteArray bytes;
	String result = TextEncoding::Utf8().BytesToString(bytes);
	ASSERT_TRUE(result == "");

	static const uint8 raw[] = { 'a', 'b' };
	bytes.Set(raw, sizeof(raw));
	result = TextEncoding::Utf8().BytesToString(bytes);
	ASSERT_TRUE(result == L"ab");

}

int main(int argc, char** argv)
{
	::testing::InitGoogleTest(&argc, argv);
	return RUN_ALL_TESTS();
}
